/*
 * オセロ盤プログラム (version 10 : 2014/11/14)
 *
 *	機能
 *		初期化の部分
 *		表示の部分 : 位置を表す記号 1 〜 8, A 〜 F
 *      入力の部分(コマを置いてみたい..)
 *			入力の指示はキーボード
 *		ゲームをしてみたい
 *          繰返し入力をする
 *			   繰返しは while で実現できるが
 *			   いつまで繰り返すか ?
 *					-> ルールでは、両方パスするまで
 *						とりあえず、サボって
 *							無限に繰り返す事にする
 *							( 終了は 'Z' を入れる )
 * v8 に向けて
 *    着手禁止の所にはうてないようにしたい
 *		「着手禁止の所」
 *			既にコマがある
 *			相手のコマをひっくりかえせない
 *    挟んだコマを自動的にコマをひっくりかえして欲い
 * ->
 *		コマを引っくり返す
 *		    自分のコマで、相手のコマを挟む
 *		イメージ : 黒で白を挟む
 *                   *oooo*  ->  ******
 *                   ^    ^
 *                 元々  打つ手
 *						上記の例は打つ手の左の状況
 *						この他に全部で 8 方向あるので、
 *                      他の 7 方向も考える
 *		取り敢えず、右横方向だけを考える
 *
 *	右方向の着手チェック
 *
 */

#include <stdio.h>

/* 表示の部分 */
/*
	print_board : 与えらたボードデータを表示する
	<<ポイント>>
		配列の引数には特殊事情がある : 後回し
*/

void print_board ( char board[8][8] ) {
  int tate;	/* 縦の添字 */ /* 追加(Copy) */
  int yoko;	/* 横の添字 */

  /* 表示機能 (移動) */

  /* ボードの内容を出力する */
  /* 8x8 の内容を全て出力(標準出力)すればよい */
  /* 配列の出力 -> for 文でやればよい */
  /* 	二次元配列なので、二重ループになる */

  /*
              A B C D E F G H
	\yoko     0 1 2 3 4 5 6 7
	tate  1 0 +---->
		  2 1 |
		  3 2 v
		  4 3
		  5 4
		  6 5
		  7 6
		  8 7
  */

  printf ( "    A   B   C   D   E  F   G   H\n" );
  printf ( "  ┏━┳━┳━┳━┳━┳━┳━┳━┓\n" );
  for ( tate = 0; tate < 8; tate++ ) {
	printf ( "%d ┃", tate + 1 ); /* 左の、この行の枠 */
	for ( yoko = 0; yoko < 8; yoko++ ) {
	  if ( board[tate][yoko] == ' ') {	/* 空き */
		printf ( "　" );
	  } else if ( board[tate][yoko] == 'o') { /* 白 */
		printf ( "◯" );
	  } else /* if board[tate][yoko] == 'x') */ { /* 黒 */
		printf ( "●" );
	  }
	  printf ( "┃" );	/* コマとコマの間の枠 */
	}
	printf ( "\n" ); /* 改行 */
	if ( tate < 8 - 1 ) {	/* 最後は、これを使わない */
	  printf ( "  ┣━╋━╋━╋━╋━╋━╋━╋━┫\n" );
	} else {	/* 最後の行に使うのはこれ */
	  printf ( "  ┗━┻━┻━┻━┻━┻━┻━┻━┛\n" );
	}
 }
}

/*
 * 初期化
 */

void init_board( char board[8][8] ) {
  int tate;	/* 縦の添字 */
  int yoko;	/* 横の添字 */

  /* ゲーム盤の初期化 */
  /* とりあえず、全部 ' ' にして、後から真ん中の４つを置く */

  /* とりあえず、全部空白 */
  for ( tate = 0; tate < 8; tate++ ) {
	  for ( yoko = 0; yoko < 8; yoko++ ) {
		board[tate][yoko] = ' ';	/* 全ての場所を空白に */
	  }
  }
  /* 真ん中の 4 つだけ特別処理 */
  board[3][3] = 'o';	/* D4 なので白 */
  board[4][4] = 'o';	/* E5 なので白 */
  board[3][4] = '*';	/* D5 なので黒 */
  board[4][3] = '*';	/* E4 なので黒 */

}

  /* ボードを作る */
  /* ボードは、8x8 の要素からなる */
  /* ボードの一つの要素には 空、黒、白のいずかが入る */
  /* 空 ' ' , 白 'o', 黒 '*'  -> 一文字 */

/*
   'A2' に黒 -> put_board_black ( 'A', '2' )
             ->  board[1][0] = '*';
*/

char get_board ( char board[8][8], char yoko, char tate ) {

  return board[tate-'1'][yoko-'A'];
}

void put_board ( char board[8][8], char yoko, char tate, char color ) {
  board[tate-'1'][yoko-'A'] = color;
  /* 本当は、ここで tate, yoko, color が正しい値かを
     チェックする必要がある (後回し)
   */
}
 
void put_board_black ( char board[8][8], char yoko, char tate ) {
  put_board ( board, yoko, tate, '*' );

  /*
	put_board_black ( 'A', '2' )
		->
	  board['2'-'1']['A'-'A'] = '*';
		->
	  board[1][0] = '*';
  */

}

void put_board_white ( char board[8][8], char yoko, char tate ) {
  put_board ( board, yoko, tate, 'o' );
}

/*
 * oposit ( char color )
 *	自分の色を指定して、相手の色を返す
 *     '*' -> 'o'
 *     'o' -> '*'
 */

char oposit ( char color ) {

  /*
  if ( color == '*' ) {
	return 'o';
  } else if ( color == 'o' ) {
	return '*';
  }
  */

  return 'o' + '*' - color;
}

/*
	横方向のチェックを行う
*/

int check_yoko ( char yoko ) {

	if ( yoko <= 'F' ) {
		if ( 'A' <= yoko ) {
			return 1;			/* 'A' <= yoko <= 'F' なので OK */
		}
	}

	return 0;		/* その他の場合は、駄目 */
}

/*
	横方向のチェックを行う
*/

int check_tate ( char tate ) {

	if ( tate <= '8' ) {
		if ( '1' <= tate ) {
			return 1;			/* '1' <= tate <= '8' なので OK */
		}
	}

	return 0;		/* その他の場合は、駄目 */
}

int check_play ( char yoko, char tate ) {

  if ( check_yoko ( yoko ) ) {
	if ( check_tate ( tate ) ) {
	  return 1;
	}
  }

  return 0;
}

int check_sandwich_play_direct_sub ( char board[8][8], char yoko, char tate, char color, char ocolor, int ydir, int tdir ) {

  yoko = yoko + ydir;
  tate = tate + tdir;
  if ( check_play ( yoko, tate ) )	{	/*右も左両方*/
		if ( get_board ( board, yoko, tate ) == color ) {
			return 1;
		} else if ( get_board ( board, yoko, tate ) == ocolor ) {
		  return check_sandwich_play_direct_sub ( board, yoko, tate, color, ocolor, ydir, tdir );
		}
	}
	return 0;
}

/*
 * check_sandwich_play_direct ( char board[8][8], char yoko, char tate, char color )
 *
 * P : 現在のゲーム盤 (board) の状態で、(yoko,tate) に color のコマが置けるかどうか ?
 *   置けたら 1 (0 以外 : C 言語の「真」) / 置けない場合は 0 (C 言語の「偽」)
 */

int check_sandwich_play_direct ( char board[8][8], char yoko, char tate, char color, int ydir, int tdir ) {

	/* 「置ける」条件 */
	if ( get_board ( board, yoko, tate ) == ' ' ) {
		/* まず、そこに他のコマがない事を確認 */
		yoko = yoko + ydir;		/* 一つ横にずらす */
		tate = tate + tdir;		/* 一つ横にずらす */
		if ( check_play ( yoko, tate ) ) {	/* 盤外でない事を確認 */
			char ocolor = oposit ( color );		/* 相手の色を入手 */
			/* color == 'o' -> '*', color == '*' -> 'o' */
			if ( get_board ( board, yoko, tate ) == ocolor ) {
				/* 一つ右が相手の色である事を確認 */
			  return check_sandwich_play_direct_sub ( board, yoko, tate, color, ocolor, ydir, tdir );
				/* 後は Q に任せる */
			}
		}
	}

	return 0;	/* どれかの条件が不成立なら、全体として不成立 (偽:0) を返す */
}

int check_sandwich_all_direct ( char board[8][8], char yoko, char tate, char color ) {
  int ydir;
  int tdir;

  for ( ydir = -1; ydir <= 1; ydir++ ) {
	for ( tdir = -1; tdir <= 1; tdir++ ) {
		if ( tate * tate + yoko * yoko != 0 ) {
		  /* tate = yoko = 0 の時だけ不成立 */
		  if ( check_sandwich_play_direct ( board, yoko, tate, color, ydir, tdir ) ) {
			return 1;
			  }
		 }
	  }
	}

  return 0;
}

